@@ -3,6 +3,8 @@
 
 package org.torproject.descriptor.impl;
 
+import static org.junit.Assert.assertEquals;
+
 import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.Microdescriptor;
 
@@ -25,7 +27,7 @@ public class MicrodescriptorImplTest {
     private static Microdescriptor createWithDefaultLines()
         throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
-      return new MicrodescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String ntorOnionKeyLine =
@@ -37,10 +39,10 @@ public class MicrodescriptorImplTest {
         throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.idLine = line;
-      return new MicrodescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
-    private byte[] buildDescriptor() {
+    private byte[] buildDescriptorBytes() {
       StringBuilder sb = new StringBuilder();
       if (this.onionKeyLines != null) {
         sb.append(this.onionKeyLines).append("\n");
@@ -53,11 +55,22 @@ public class MicrodescriptorImplTest {
       }
       return sb.toString().getBytes();
     }
+
+    private Microdescriptor buildDescriptor(
+        boolean failUnrecognizedDescriptorLines)
+        throws DescriptorParseException {
+      byte[] descriptorBytes = this.buildDescriptorBytes();
+      return new MicrodescriptorImpl(descriptorBytes,
+          new int[] { 0, descriptorBytes.length },
+          failUnrecognizedDescriptorLines);
+    }
   }
 
   @Test()
   public void testDefaults() throws DescriptorParseException {
-    DescriptorBuilder.createWithDefaultLines();
+    Microdescriptor micro = DescriptorBuilder.createWithDefaultLines();
+    assertEquals("ER1AC4KqT//o3pJDrqlmej5G2qW1EQYEr/IrMQHNc6I",
+        micro.getDigestSha256Base64());
   }
 
   @Test(expected = DescriptorParseException.class)
