@@ -10,6 +10,7 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertTrue;
 
 import org.torproject.descriptor.BandwidthHistory;
+import org.torproject.descriptor.BridgeServerDescriptor;
 import org.torproject.descriptor.DescriptorParseException;
 import org.torproject.descriptor.ServerDescriptor;
 
@@ -37,7 +38,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.routerLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String bandwidthLine = "bandwidth 51200 51200 53470";
@@ -46,7 +47,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.bandwidthLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String platformLine = "platform Tor 0.2.2.35 "
@@ -56,7 +57,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.platformLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String publishedLine = "published 2012-01-01 04:03:19";
@@ -65,7 +66,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.publishedLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String fingerprintLine = "opt fingerprint D873 3048 FC8E "
@@ -75,7 +76,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.fingerprintLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String hibernatingLine = null;
@@ -84,7 +85,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.hibernatingLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String uptimeLine = "uptime 48";
@@ -93,7 +94,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.uptimeLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String onionKeyLines = "onion-key\n"
@@ -108,7 +109,7 @@ public class ServerDescriptorImplTest {
         String lines) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.onionKeyLines = lines;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String signingKeyLines = "signing-key\n"
@@ -123,7 +124,7 @@ public class ServerDescriptorImplTest {
         String lines) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.signingKeyLines = lines;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String onionKeyCrosscertLines = null;
@@ -132,7 +133,7 @@ public class ServerDescriptorImplTest {
         String lines) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.onionKeyCrosscertLines = lines;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String ntorOnionKeyCrosscertLines = null;
@@ -141,7 +142,7 @@ public class ServerDescriptorImplTest {
         String lines) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.ntorOnionKeyCrosscertLines = lines;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String exitPolicyLines = "reject *:*";
@@ -150,7 +151,7 @@ public class ServerDescriptorImplTest {
         String lines) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.exitPolicyLines = lines;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String contactLine = "contact Random Person <nobody AT "
@@ -160,7 +161,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.contactLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String familyLine = null;
@@ -169,7 +170,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.familyLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String readHistoryLine = null;
@@ -178,7 +179,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.readHistoryLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String writeHistoryLine = null;
@@ -187,7 +188,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.writeHistoryLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String eventdnsLine = null;
@@ -196,7 +197,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.eventdnsLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String cachesExtraInfoLine = null;
@@ -205,7 +206,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.cachesExtraInfoLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String extraInfoDigestLine = "opt extra-info-digest "
@@ -215,7 +216,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.extraInfoDigestLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String hiddenServiceDirLine = "opt hidden-service-dir";
@@ -224,7 +225,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.hiddenServiceDirLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String protocolsLine = null;
@@ -233,7 +234,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.protocolsLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String protoLine = "proto Cons=1-2 Desc=1-2 DirCache=1 HSDir=1 "
@@ -243,7 +244,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.protoLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String allowSingleHopExitsLine = null;
@@ -253,7 +254,7 @@ public class ServerDescriptorImplTest {
         throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.allowSingleHopExitsLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String ipv6PolicyLine = null;
@@ -262,7 +263,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.ipv6PolicyLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String ntorOnionKeyLine = null;
@@ -271,7 +272,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.ntorOnionKeyLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String tunnelledDirServerLine = null;
@@ -280,7 +281,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.tunnelledDirServerLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String routerSignatureLines = "router-signature\n"
@@ -294,7 +295,7 @@ public class ServerDescriptorImplTest {
         String line) throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.routerSignatureLines = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
     private String unrecognizedLine = null;
@@ -304,8 +305,7 @@ public class ServerDescriptorImplTest {
         throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.unrecognizedLine = line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(),
-          failUnrecognizedDescriptorLines);
+      return db.buildDescriptor(failUnrecognizedDescriptorLines);
     }
 
     private byte[] nonAsciiLineBytes = null;
@@ -315,8 +315,7 @@ public class ServerDescriptorImplTest {
             throws DescriptorParseException {
       DescriptorBuilder db = new DescriptorBuilder();
       db.nonAsciiLineBytes = lineBytes;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(),
-          failUnrecognizedDescriptorLines);
+      return db.buildDescriptor(failUnrecognizedDescriptorLines);
     }
 
     private String identityEd25519Lines = null;
@@ -332,10 +331,10 @@ public class ServerDescriptorImplTest {
       db.identityEd25519Lines = identityEd25519Lines;
       db.masterKeyEd25519Line = masterKeyEd25519Line;
       db.routerSigEd25519Line = routerSigEd25519Line;
-      return new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+      return db.buildDescriptor(true);
     }
 
-    private byte[] buildDescriptor() {
+    private byte[] buildDescriptorBytes() {
       StringBuilder sb = new StringBuilder();
       if (this.routerLine != null) {
         sb.append(this.routerLine).append("\n");
@@ -446,13 +445,21 @@ public class ServerDescriptorImplTest {
       }
       return sb.toString().getBytes();
     }
+
+    private ServerDescriptorImpl buildDescriptor(
+        boolean failUnrecognizedDescriptorLines)
+        throws DescriptorParseException {
+      byte[] descriptorBytes = this.buildDescriptorBytes();
+      return new RelayServerDescriptorImpl(descriptorBytes,
+          new int[] { 0, descriptorBytes.length },
+          failUnrecognizedDescriptorLines);
+    }
   }
 
   @Test()
   public void testSampleDescriptor() throws DescriptorParseException {
     DescriptorBuilder db = new DescriptorBuilder();
-    ServerDescriptor descriptor =
-        new RelayServerDescriptorImpl(db.buildDescriptor(), true);
+    ServerDescriptor descriptor = db.buildDescriptor(true);
     assertEquals("saberrider2008", descriptor.getNickname());
     assertEquals("94.134.192.243", descriptor.getAddress());
     assertEquals(9001, (int) descriptor.getOrPort());
@@ -472,7 +479,7 @@ public class ServerDescriptorImplTest {
     assertEquals(51200, (int) descriptor.getBandwidthBurst());
     assertEquals(53470, (int) descriptor.getBandwidthObserved());
     assertEquals("1469D1550738A25B1E7B47CDDBCD7B2899F51B74",
-        descriptor.getExtraInfoDigest());
+        descriptor.getExtraInfoDigestSha1Hex());
     assertEquals(Arrays.asList(new Integer[] {2}),
         descriptor.getHiddenServiceDirVersions());
     assertEquals("Random Person <nobody AT example dot com>",
@@ -487,6 +494,10 @@ public class ServerDescriptorImplTest {
     assertFalse(descriptor.getCachesExtraInfo());
     assertFalse(descriptor.getAllowSingleHopExits());
     assertTrue(descriptor.getUnrecognizedLines().isEmpty());
+    assertEquals("a9635dd801ad98dac43aff49baa2dbbaf050222d",
+        descriptor.getDigestSha1Hex());
+    assertEquals("kvdJKQ6R9i8x1nDqJZ34JFWsu6TquLqQy54nheSWrOY",
+        descriptor.getDigestSha256Base64());
   }
 
   @Test(expected = DescriptorParseException.class)
@@ -828,7 +839,7 @@ public class ServerDescriptorImplTest {
         .createWithExtraInfoDigestLine("extra-info-digest "
         + "1469D1550738A25B1E7B47CDDBCD7B2899F51B74");
     assertEquals("1469D1550738A25B1E7B47CDDBCD7B2899F51B74",
-        descriptor.getExtraInfoDigest());
+        descriptor.getExtraInfoDigestSha1Hex());
   }
 
   @Test(expected = DescriptorParseException.class)
@@ -858,7 +869,7 @@ public class ServerDescriptorImplTest {
       throws DescriptorParseException {
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExtraInfoDigestLine(null);
-    assertNull(descriptor.getExtraInfoDigest());
+    assertNull(descriptor.getExtraInfoDigestSha1Hex());
   }
 
   @Test()
@@ -871,7 +882,7 @@ public class ServerDescriptorImplTest {
         extraInfoDigest, additionalExtraInfoDigest);
     ServerDescriptor descriptor = DescriptorBuilder
         .createWithExtraInfoDigestLine(extraInfoDigestLine);
-    assertEquals(extraInfoDigest, descriptor.getExtraInfoDigest());
+    assertEquals(extraInfoDigest, descriptor.getExtraInfoDigestSha1Hex());
   }
 
   @Test()
@@ -1027,14 +1038,14 @@ public class ServerDescriptorImplTest {
         descriptor.getExitPolicyLines());
   }
 
-  @Test()
-  public void testRouterSignatureOpt()
-      throws DescriptorParseException {
-    DescriptorBuilder.createWithRouterSignatureLines("opt "
-        + "router-signature\n"
+  @Test(expected = DescriptorParseException.class)
+  public void testEndSignatureFourDashes() throws DescriptorParseException {
+    DescriptorBuilder.createWithRouterSignatureLines("router-signature\n"
         + "-----BEGIN SIGNATURE-----\n"
-        + "crypto lines are ignored anyway\n"
-        + "-----END SIGNATURE-----");
+        + "o4j+kH8UQfjBwepUnr99v0ebN8RpzHJ/lqYsTojXHy9kMr1RNI9IDeSzA7PSqT"
+        + "uV\n4PL8QsGtlfwthtIoZpB2srZeyN/mcpA9fa1JXUrt/UN9K/+32Cyaad7h0n"
+        + "HE6Xfb\njqpXDpnBpvk4zjmzjjKYnIsUWTnADmu0fo3xTRqXi7g=\n"
+        + "-----END SIGNATURE----");
   }
 
   @Test(expected = DescriptorParseException.class)
@@ -1245,11 +1256,11 @@ public class ServerDescriptorImplTest {
         "write-history 2012-01-01 03:51:44 (900 ");
   }
 
-  @Test(expected = DescriptorParseException.class)
-  public void testWriteHistoryTrailingNumber()
+  @Test()
+  public void testWriteHistoryExtraArg()
       throws DescriptorParseException {
     DescriptorBuilder.createWithWriteHistoryLine("write-history "
-        + "2012-01-01 03:51:44 (900 s) 4345856 1");
+        + "2012-01-01 03:51:44 (900 s) 4345856 bin_size=1024");
   }
 
   @Test()
@@ -1652,6 +1663,14 @@ public class ServerDescriptorImplTest {
         + ROUTER_SIG_ED25519_LINE);
   }
 
+  @Test(expected = DescriptorParseException.class)
+  public void testEd25519FollowedbyUnrecognizedLine()
+      throws DescriptorParseException {
+    DescriptorBuilder.createWithEd25519Lines(IDENTITY_ED25519_LINES,
+        MASTER_KEY_ED25519_LINE, ROUTER_SIG_ED25519_LINE
+        + "\nunrecognized-line 1");
+  }
+
   private static final String ONION_KEY_CROSSCERT_LINES =
       "onion-key-crosscert\n"
       + "-----BEGIN CROSSCERT-----\n"
@@ -1704,5 +1723,31 @@ public class ServerDescriptorImplTest {
         NTOR_ONION_KEY_CROSSCERT_LINES + "\n"
         + NTOR_ONION_KEY_CROSSCERT_LINES);
   }
+
+  @Test()
+  public void testBridgeDescriptorDigestsRouterDigestLines()
+      throws DescriptorParseException {
+    DescriptorBuilder db = new DescriptorBuilder();
+    String digestSha1Hex = "A9635DD801AD98DAC43AFF49BAA2DBBAF050222D";
+    String digestSha256Base64 = "kvdJKQ6R9i8x1nDqJZ34JFWsu6TquLqQy54nheSWrOY";
+    db.routerSignatureLines = "router-digest-sha256 " + digestSha256Base64
+        + "\nrouter-digest " + digestSha1Hex;
+    byte[] descriptorBytes = db.buildDescriptorBytes();
+    BridgeServerDescriptor descriptor = new BridgeServerDescriptorImpl(
+        descriptorBytes, new int[] { 0, descriptorBytes.length }, true);
+    assertEquals(digestSha1Hex, descriptor.getDigestSha1Hex());
+    assertEquals(digestSha256Base64, descriptor.getDigestSha256Base64());
+  }
+
+  @Test()
+  public void testBridgeDescriptorDigestsNoRouterDigestLines()
+      throws DescriptorParseException {
+    DescriptorBuilder db = new DescriptorBuilder();
+    byte[] descriptorBytes = db.buildDescriptorBytes();
+    BridgeServerDescriptor descriptor = new BridgeServerDescriptorImpl(
+        descriptorBytes, new int[] { 0, descriptorBytes.length }, true);
+    assertNull(descriptor.getDigestSha1Hex());
+    assertNull(descriptor.getDigestSha256Base64());
+  }
 }
 
